@@ -1,7 +1,7 @@
 use clean_insights_sdk::campaign::{Campaign, EventAggregationRule};
 use clean_insights_sdk::cleaninsights::CleanInsights;
 use clean_insights_sdk::configuration::Configuration;
-use clean_insights_sdk::consents::{Feature, Consent};
+use clean_insights_sdk::consents::{Feature, Consent, Consents};
 use clean_insights_sdk::default_store::DefaultStore;
 use clean_insights_sdk::event::Event;
 use clean_insights_sdk::insights::Insights;
@@ -9,6 +9,7 @@ use clean_insights_sdk::store::Store;
 use clean_insights_sdk::visit::Visit;
 
 use std::collections::HashMap;
+use std::error::Error;
 use std::fs::{remove_file, File};
 use std::io::BufReader;
 use std::mem::transmute;
@@ -29,7 +30,7 @@ impl Tests {
 
     const STORE_FILE: &'static str = "tests/cleaninsights.json";
 
-    fn setup(strengthen_anonymity: bool) -> Tests {
+    fn setup(strengthen_anonymity: bool, max_age_of_old_data: Option<i64>) -> Tests {
         remove_file(Path::new(Tests::STORE_FILE)).unwrap_or(());
 
         let mut campaigns = HashMap::new();
@@ -48,7 +49,7 @@ impl Tests {
             site_id: 1,
             timeout: 1,
             max_retry_delay: 1,
-            max_age_of_old_data: 1,
+            max_age_of_old_data: max_age_of_old_data.unwrap_or(1),
             persist_every_n_times: 1,
             server_side_anonymous_usage: false,
             debug: true,
@@ -117,14 +118,14 @@ pub struct CleanInsightsTest {
 
 #[test]
 fn conf() {
-    let t = Tests::setup(false);
+    let t = Tests::setup(false, None);
 
     assert_eq!(t.conf, t.ci.conf);
 }
 
 #[test]
 fn deny_consent() {
-    let mut t = Tests::setup(false);
+    let mut t = Tests::setup(false, None);
 
     t.ci.deny_feature(Feature::Lang);
     t.ci.deny_feature(Feature::Ua);
@@ -148,7 +149,7 @@ fn deny_consent() {
 
 #[test]
 fn grant_consent() {
-    let mut t = Tests::setup(false);
+    let mut t = Tests::setup(false, None);
 
     t.ci.grant_feature(Feature::Lang);
     t.ci.grant_feature(Feature::Ua);
@@ -172,7 +173,7 @@ fn grant_consent() {
 
 #[test]
 fn grant_consent_strengthened_anonymity() {
-    let mut t = Tests::setup(true);
+    let mut t = Tests::setup(true, None);
 
     t.ci.grant_feature(Feature::Lang);
     t.ci.grant_feature(Feature::Ua);
@@ -199,7 +200,7 @@ fn grant_consent_strengthened_anonymity() {
 #[test]
 #[serial]
 fn persistence() {
-    let mut t = Tests::setup(false);
+    let mut t = Tests::setup(false, None);
 
     t.ci.grant_feature(Feature::Lang);
     t.ci.grant_feature(Feature::Ua);
@@ -263,7 +264,7 @@ fn persistence() {
 #[test]
 #[serial]
 fn persistence_strengthened_anonymity() {
-    let mut t = Tests::setup(true);
+    let mut t = Tests::setup(true, None);
 
     t.ci.grant_feature(Feature::Lang);
     t.ci.grant_feature(Feature::Ua);
@@ -339,7 +340,7 @@ fn persistence_strengthened_anonymity() {
 
 #[test]
 fn serialize_insights() {
-    let insights = Insights {
+    let mut insights = Insights {
         idsite: 1,
         lang: Some("foo".to_string()),
         ua: Some("bar".to_string()),
@@ -348,4 +349,102 @@ fn serialize_insights() {
     };
 
     assert_eq!(serde_json::to_string(&insights).unwrap(), "{\"idsite\":1,\"lang\":\"foo\",\"ua\":\"bar\",\"visits\":[],\"events\":[]}");
+
+    let mut v: Vec<Visit> = vec![Visit {
+        scene_path: vec!["foo".to_string(), "bar".to_string()],
+        campaign_id: "test".to_string(),
+        times: 1,
+        first: DateTime::parse_from_rfc3339("2022-01-01T00:00:00-00:00").unwrap().with_timezone(&Utc),
+        last: DateTime::parse_from_rfc3339("2022-01-02T00:00:00-00:00").unwrap().with_timezone(&Utc)
+    }];
+
+    let mut e: Vec<Event> = vec![Event {
+        category: "foo".to_string(),
+        action: "bar".to_string(),
+        name: Some("baz".to_string()),
+        value: Some(6.66),
+        campaign_id: "test".to_string(),
+        times: 1,
+        first: DateTime::parse_from_rfc3339("2022-01-01T00:00:00-00:00").unwrap().with_timezone(&Utc),
+        last: DateTime::parse_from_rfc3339("2022-01-02T00:00:00-00:00").unwrap().with_timezone(&Utc)
+    }];
+
+    let mut store: Box<dyn Store> = Box::new(TestStore::new());
+    store.visits_mut().append(&mut v);
+    store.events_mut().append(&mut e);
+
+    let t = Tests::setup(false, Some(30000));
+
+    insights = Insights::new(&t.conf, &mut *store, &vec![], &None);
+
+    assert_eq!(serde_json::to_string(&insights).unwrap(),
+               "{\"idsite\":1,\"visits\":[{\"action_name\":\"foo/bar\",\"campaign_id\":\"test\",\"times\":1,\"period_start\":1640995200,\"period_end\":1641081600}],\"events\":[{\"category\":\"foo\",\"action\":\"bar\",\"name\":\"baz\",\"value\":6.66,\"campaign_id\":\"test\",\"times\":1,\"period_start\":1640995200,\"period_end\":1641081600}]}")
+}
+
+#[test]
+fn default_store_send() {
+    let store = DefaultStore::new(
+        "/tmp", |message| debug(&message));
+
+    assert!(store.send("".to_string(), &"http://example.org/test".to_string(), 1.5 as u64).is_err());
+}
+
+fn debug(message: &str) {
+    eprintln!("[CleanInsightsSDK Test] {}", message);
+}
+
+#[derive(Debug)]
+struct TestStore {
+
+    consents: Consents,
+
+    visits: Vec<Visit>,
+
+    events: Vec<Event>,
+}
+
+impl TestStore {
+
+    fn new() -> TestStore {
+        TestStore {
+            consents: Consents { features: Default::default(), campaigns: Default::default() },
+            visits: vec![],
+            events: vec![] }
+    }
+}
+
+
+impl Store for TestStore {
+
+    fn consents(&self) -> &Consents {
+        &self.consents
+    }
+
+    fn consents_mut(&mut self) -> &mut Consents {
+        &mut self.consents
+    }
+
+    fn visits(&self) -> &Vec<Visit> {
+        &self.visits
+    }
+
+    fn visits_mut(&mut self) -> &mut Vec<Visit> {
+        &mut self.visits
+    }
+
+    fn events(&self) -> &Vec<Event> {
+        &self.events
+    }
+
+    fn events_mut(&mut self) -> &mut Vec<Event> {
+        &mut self.events
+    }
+
+    fn persist(&self) -> Result<(), Box<dyn Error>> {
+        Ok(())
+    }
+
+    fn send(&self, _data: String, _server: &String, _timeout: u64) -> Result<(), Box<dyn Error>> {
+        Ok(())
+    }
 }
\ No newline at end of file
